# SDM NVC community analyses

# This script uses GIS spatial data stored as .tif and .shp files in the 
# DATA/GIS_folder, and stores SDM model results as .RDS files in DATA/RDS folder.
# Prediction maps from the sdm::predict() function are then saved in DATA/SDM_preds

# Utility functions ####
predict_min_max <- function(x, n=5){
  # Fix any rounding errors above and below 0 to 1 on some maps
  for(i in 1:n){
    x[[i]][x[[i]] > 1] <- 1
    x[[i]][x[[i]] < 0] <- 0
    x[[i]][1,1] <- 1 # Fix scalebar max to 1
    x[[i]][1,2] <- 0 # Fix scalebar min to 1
  }
  return(x)
}

plot_obs_pred <- function(NVC_pred, NVC_obs, method = c("GLM", "SVM", "RF", "MaxEnt", "MaxLike")){
  # Multiple plots on same scale etc ####
  par(mfrow=c(2,3))
  for (i in 1:5){
    plot(NVC_pred[[i]], axes = FALSE, box = FALSE,
         main = method[i])
    plot(NVC_obs, add=TRUE, pch=22)
    
  }  
  par(mfrow=c(1,1))
}



library(sf)
library(raster)
library(sdm)
library(dplyr)
library(tidyr)
library(ggplot2)

# Env data ####
soil_1k <- raster("DATA/GIS_data/Rasters/soil_1k.tif") # Not in repository
DEM_1k <- raster("DATA/GIS_data/Rasters/DEM_1k.tif") 
aspect_1k <- raster("DATA/GIS_data/Rasters/aspect_1k.tif")
slope_1k <- raster("DATA/GIS_data/Rasters/slope_1k.tif")
EVI_1k <- raster("DATA/GIS_data/Rasters/EVI_1k.tif")
LCM2015_1k <- raster("DATA/GIS_data/Rasters/LCM2015_1k.tif") # Not in repository
rainfall_1k <- raster("DATA/GIS_data/Rasters/rainfall.tif")
tempmin_summer_MET <- raster("DATA/GIS_data/Rasters/tempmin_summer_MET.tif")
tempmin_winter_MET <- raster("DATA/GIS_data/Rasters/tempmin_winter_MET.tif")
tempmax_summer_MET <- raster("DATA/GIS_data/Rasters/tempmax_summer_MET.tif")
tempmax_winter_MET <- raster("DATA/GIS_data/Rasters/tempmax_winter_MET.tif")
LANDSAT_EVI <- raster("DATA/GIS_data/RASTERS/EVI_1k.tif")

full_stack_1k <- stack(c(aspect_1k, DEM_1k, slope_1k, LCM2015_1k, EVI_1k, soil_1k,
                         rainfall_1k, tempmin_summer_MET, tempmin_winter_MET,
                         tempmax_summer_MET, tempmax_winter_MET))


# Community data ####
NVC_names <- c("CG9", "CG10", "H8", "H10", "H18", "H19", "H21", "M1", "M2", "M3",
               "M4", "M5", "M9", "M11", "M16", "M17", "M18", "M20", "M21", "M25",
               "M27", "M29", "M37", "U7", "U10", "U16", "U17", "U21", "W9", "W19")


process_data <- function(nvc_list){
  for(nvc_no in 1:length(nvc_list)){
    print(paste0("Processing NVC ", nvc_list[nvc_no], "..."))
    NVC <- st_read(paste0("DATA/GIS_data/Communities/Shapefiles/cropped/",
                          nvc_list[nvc_no], ".shp" ))
    NVC <- st_transform(NVC, 27700)
    NVC_sp <- as(NVC, "Spatial")
    NVC_sp$NVC <- 1
    set.seed(105)
    
    NVC_sdmData <- sdmData(NVC ~  aspect_1k + DEM_1k + slope_1k + LCM2015_1k + EVI_1k +
                             soil_1k + rainfall + tempmin_summer_MET +
                             tempmin_winter_MET + tempmax_summer_MET +
                             tempmax_winter_MET,
                           train = NVC_sp, predictors = full_stack_1k,
                           bg = list(n=100, method = 'gRandom', remove = TRUE))
    set.seed(006)
    NVC_sdm <- sdm(NVC ~ aspect_1k + DEM_1k + slope_1k + LCM2015_1k + EVI_1k + soil_1k +
                     rainfall + tempmin_summer_MET + tempmin_winter_MET +
                     tempmax_summer_MET + tempmax_winter_MET,
                   data = NVC_sdmData,
                   methods = c('glm', 'svm', 'rf', 'entropy', 'maxlike'),
                   replication = 'boot', n = 10)
    predict_NVC_mean <- predict(NVC_sdm, full_stack_1k, overwrite = TRUE,
                                mean = TRUE,
                                filename = paste0("DATA/SDM_preds/",
                                                  nvc_list[nvc_no],
                                                  "_mean_predict.img"))
    predict_NVC_mean <- predict_min_max(predict_NVC_mean)
    saveRDS(NVC, paste0("DATA/RDS/", nvc_list[nvc_no], ".RDS"))
    saveRDS(NVC_sp, paste0("DATA/RDS/", nvc_list[nvc_no], "_sp.RDS"))
    saveRDS(NVC_sdmData, paste0("DATA/RDS/", nvc_list[nvc_no], "_sdmData.RDS"))
    saveRDS(NVC_sdm, paste0("DATA/RDS/", nvc_list[nvc_no], "_sdm.RDS"))
    saveRDS(predict_NVC_mean, paste0("DATA/RDS/", nvc_list[nvc_no], "_predict.RDS"))
    }
}
process_data(NVC_names)

# Compare model performance with number of occurrence records ####
# Occurrences. Note, this is lower than the number of records as duplicate 10kms
nvc_perf <- data.frame(matrix(nrow=length(NVC_names), ncol=16))
colnames(nvc_perf) <- c("count","AUC_GLM","AUC_MaxEnt","AUC_MaxLike","AUC_RF",
                        "AUC_SVM","TSS_GLM","TSS_MaxEnt","TSS_MaxLike","TSS_RF",
                        "TSS_SVM","KPP_GLM","KPP_MaxEnt","KPP_MaxLike","KPP_RF",
                        "KPP_SVM")
for(nvc_no in 1:length(NVC_names)){
  NVC <- st_read(paste0("DATA/GIS_data/Communities/Shapefiles/cropped/",
                        NVC_names[nvc_no], ".shp" ))
  NVC <- st_transform(NVC, 27700)
  NVC_cnt <- nrow(distinct(NVC))
  nvc_perf[nvc_no, 1] <- NVC_cnt
  
  # Model performance data AUC and TSS
  NVC_sdm <- readRDS(paste0("DATA/RDS/", NVC_names[nvc_no], "_sdm.RDS"))
  NVC_eval <- getEvaluation(NVC_sdm, stat = c('TSS', 'Kappa', 'AUC'), opt = 1)
  methods <- rep(c('GLM', 'SVM', 'RF', 'MaxEnt', 'MaxLike'), each = 10)
  NVC_eval <- cbind(NVC_eval, methods)
  
  NVC_eval_tbl <- NVC_eval %>%
    rename(`Model` = methods) %>% 
    group_by(`Model`) %>% 
    summarise(AUC = mean(AUC), TSS = mean(TSS), Kappa = mean(Kappa)) %>% 
    pivot_wider(names_from=1, values_from=2:4)
  
  this_nvc_perf <- c(NVC_cnt, NVC_eval_tbl)
  nvc_perf[nvc_no,] <- this_nvc_perf
}

nvc_perf_lng <- nvc_perf %>% 
  pivot_longer(cols=2:16) %>% 
  mutate(metric = substr(name, 1, 3), model = substr(name, 5, nchar(name)))

# Check whether there is any relationship between number of occurrences and
# model performance (no evidence of any effects).
nvc_perf_tss <- filter(nvc_perf_lng, metric == "TSS") %>% 
  group_by(count) %>% 
  summarise(mean_metric = mean(value))
ggplot(nvc_perf_tss, aes(x = count, y = mean_metric)) +
  geom_point() +
  xlab("Number of 10 km occurrence records") +
  ylab("TSS") +
  theme_classic()
summary(lm(mean_metric ~ count, data = nvc_perf_tss))

nvc_perf_auc <- filter(nvc_perf_lng, metric == "AUC") %>% 
  group_by(count) %>% 
  summarise(mean_metric = mean(value))
ggplot(nvc_perf_auc, aes(x = count, y = mean_metric)) +
  geom_point() +
  xlab("Number of 10 km occurrence records") +
  ylab("AUC") +
  theme_classic()
summary(lm(mean_metric ~ count, data = nvc_perf_auc))

nvc_perf_kappa <- filter(nvc_perf_lng, metric == "KPP") %>% 
  group_by(count) %>% 
  summarise(mean_metric = mean(value))
ggplot(nvc_perf_kappa, aes(x = count, y = mean_metric)) +
  geom_point() +
  xlab("Number of 10 km occurrence records") +
  ylab("Kappa") +
  theme_classic()
summary(lm(mean_metric ~ count, data = nvc_perf_kappa))

# Mean model performance
library(dplyr)
nvc_list <- c("CG9", "CG10", "H8", "H10", "H18", "H19", "H21", "M1", "M2", "M3",
              "M4", "M5", "M9", "M11", "M16", "M17", "M18", "M20", "M21", "M25",
              "M27", "M29", "M37", "U7", "U10", "U16", "U17", "U21", "W9", "W19")
NVC_var_df <- NULL #data.frame(matrix(nrow=10*length(nvc_list), ncol=5))
for(nvc_no in 1:length(nvc_list)){
  NVC_sdm <- readRDS(paste0("DATA/RDS/", nvc_list[nvc_no], "_sdm.RDS"))
  NVC_var <- getVarImp(NVC_sdm, wtest = 'training', id=1:50)
  NVC_var_plt <- plot(NVC_var, 'cor')
  NVC_var_dta <- NVC_var_plt$data
  NVC_var_dta <- mutate(NVC_var_dta, nvc = nvc_list[nvc_no])
  NVC_var_df <- rbind(NVC_var_df, NVC_var_dta)
}

NVC_var_df %>% 
  group_by(variables) %>% 
  summarise(mean = mean(corTest),
            sd = sd(corTest),
            n = n(),
            se = sd / sqrt(n),
            ci95 = se * qt(.975, n - 1)) %>% 
  ggplot(aes(x = variables, y = mean)) +
    geom_bar(stat = "identity", fill = "grey60") +
  geom_errorbar(aes(ymin = mean - ci95, ymax = mean + ci95),
                size = 0.3,
                width = 0.2) +
  scale_x_discrete(labels = c("aspect","elevation","land cover",
                              "annual rainfall", "EVI",
                              "slope","soil type","summer max temp",
                              "winter max temp","summer min temp",
                              "winter min temp")) +
  ylab("Variable importance (1 - cor)") +
  theme_classic() +
  theme(axis.title.x = element_blank(),
        axis.text.x  = element_text(angle=60, vjust=0.5, size=10)) 

# Add info on performance statistics per NVC
nvc_perf_lng <- nvc_perf_lng %>% 
  bind_cols(rep(nvc_list, each=15)) %>% 
  rename(nvc = ...6)

# Performance boxplots overall
ggplot(nvc_perf_lng, aes(x=model, y=value)) +
  geom_boxplot() +
  facet_wrap( ~ metric, labeller=labeller(metric = c(AUC = "a) AUC",
                                                     KPP = "b) Kappa",
                                                     TSS = "c) TSS"))) +
  ylab("Model performance") +
  theme_classic() +
  theme(axis.title.x = element_blank())

library(cowplot)
p1 <- nvc_perf_lng %>% 
  filter(metric == "AUC") %>% 
  ggplot(aes(x = model, y = value)) +
    geom_boxplot() +
    ylim(c(0, 1)) +
    ylab("AUC") +
    theme_classic() +
    theme(axis.title.x = element_blank())
p2 <- nvc_perf_lng %>% 
  filter(metric == "TSS") %>% 
  ggplot(aes(x = model, y = value)) +
  geom_boxplot() +
  ylim(c(0, 1)) +
  ylab("TSS") +
  theme_classic() +
  theme(axis.title.x = element_blank())
p3 <- nvc_perf_lng %>% 
  filter(metric == "KPP") %>% 
  ggplot(aes(x = model, y = value)) +
  geom_boxplot() +
  ylim(c(0, 1)) +
  ylab("Kappa") +
  theme_classic() +
  theme(axis.title.x = element_blank())
plot_grid(p1, p2, p3, labels = c("a", "b", "c"))


# Check on overall performance summaries for example communities
# High scores: U10, U17, W19, H19, M11
# Median scores: CG10, M1, M3, M4, H18
# Low scores: H8, M16, M21, M27,
nvc_perf_lng %>% 
  filter(metric == "TSS") %>% 
  arrange(desc(value)) %>% 
  View()

# Extract maps for Figs 5, 6 and 7 of paper
plot_obs_pred <- function(NVC_pred, NVC_obs, method = c("GLM", "SVM", "RF", "MaxEnt", "MaxLike")){
  # Multiple plots on same scale etc ####
  par(mfrow=c(2,3))
  for (i in 1:5){
    plot(NVC_pred[[i]], axes = FALSE, box = FALSE,
         main = method[i])
    plot(NVC_obs, add=TRUE, pch=22, cex=0.5)
    
  }  
  par(mfrow=c(1,1))
}
# Fig. 5 U17
NVC_sp <- readRDS(paste0("DATA/RDS/U17_sp.RDS"))
predict_NVC_mean <- readRDS("DATA/RDS/U17_predict.RDS")
plot_obs_pred(predict_NVC_mean, NVC_sp)
# Fig. 6 H18
NVC_sp <- readRDS(paste0("DATA/RDS/H18_sp.RDS"))
predict_NVC_mean <- readRDS("DATA/RDS/H18_predict.RDS")
plot_obs_pred(predict_NVC_mean, NVC_sp)
# Fig. 7 M27
NVC_sp <- readRDS(paste0("DATA/RDS/M27_sp.RDS"))
predict_NVC_mean <- readRDS("DATA/RDS/M27_predict.RDS")
plot_obs_pred(predict_NVC_mean, NVC_sp)
